//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "_DRAW_2D.h"

//////////////////////////////////////////////////////////////////////////////
int keyExit = 0; 

//////////////////////////////////////////////////////////////////////////////////////////
inline tfloat sq(tfloat t) { return t * t; }; 
inline tfloat dist(tfloat x1, tfloat y1, tfloat x2, tfloat y2) { return sqrt(sq(x2 - x1) + sq(y2 - y1)); };
inline tfloat force(tfloat r, tfloat x1, tfloat y1, tfloat x2, tfloat y2)
{
  tfloat d = dist(x1, y1, x2, y2); if (d > r) return 0; else return (1 - d / r);
};

//////////////////////////////////////////////////////////////////////////////
#define count 30000

tfloat speed = 0.5, radius = 1.25, reaction = 0.3, falloff = 0.975, border = 1, scale = 1;

void keyPressed()                           //  [ ] 
{
  keyExit = (II.KeyUp[_Esc_]);
  II.Reset();
};

//////////////////////////////////////////////////////////////////////////////
class c_circ
{
  public:
  tfloat rad, bord, x, y, dx, dy, xx, yy, ddx, ddy;
  void reset()
  {
     rad  = fRandom(radius * 0.35, radius * 1.15) * scale; bord = rad * border;
     xx = x = fRandom(bord, DrawXL - bord);
     yy = y = fRandom(bord, DrawYL - bord);
     tfloat d = fRandom(3, 5) * speed * scale;
     ddx = 0; dx  = d * RandomSign();
     ddy = 0; dy  = d * RandomSign();
  };
  void regenerate()
  { 
    x += dx; y += dy;
    if ((x > (DrawXL - bord)) || (x < bord)) dx = - dx; else if (Random(37) == 11) dx = - dx;
    if ((y > (DrawYL - bord)) || (y < bord)) dy = - dy; else if (Random(37) == 11) dy = - dy;  
    tfloat d = dist(x, y, xx, yy);
    ddx += (x - xx) / d * reaction;
    ddy += (y - yy) / d * reaction;
    ddx *= falloff; ddy *= falloff;
    xx += ddx; yy += ddy;
  };
  void draw(int i, float r, dwRGB c, unsigned m)
  {
    DrawCircle(xx, yy, r * 5, RGBprocent(c, 25), m);
    DrawCircle(xx, yy, r, c, m);
  };
  void draw(int i, tfloat px1, tfloat py1, tfloat px2, tfloat py2, tfloat px3, tfloat py3, tfloat r)
  {
    r *= scale;
    tfloat f1 = force(r, xx, yy, px1, py1),
           f2 = force(r, xx, yy, px2, py2),
           f3 = force(r, xx, yy, px3, py3),
           f = f1 + f2 + f3;
    if (f > 0.001)
    { dwRGB c1 = RGBfmultV(RGBis(25, 10, 0), 2.75 * f1),
            c2 = RGBfmultV(RGBis(10, 20, 0), 2.75 * f2),
            c3 = RGBfmultV(RGBis(0, 10, 25), 2.75 * f3); 
      draw(i, rad * f, RGBadd(RGBadd(c1, c2), c3), modeADD); 
    };
  };
};

/////////////////////////////////////////////////////////////////////////////////////////////////
c_circ circ[count],pcirc[3];

/////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsInit()
{ 
  FC_FpsSet(AUTO);//    
  FC_DrawPaint = Yes;          //  fps   DrawPaint();
  RandomSeed();
  Draw2D_Init(); ForceActivate(); ScreenLoadWait();
  OnKeyUpDo(keyPressed);       //     
  keyPressed();                //     ...

  scale = (tfloat)DrawXR / 1600; 
  
  for (int i = 0; i < count; i++) circ[i].reset();
  
  speed = 2; radius = 150; 
  for (int i = 0; i < 3; i++) pcirc[i].reset();/**/
};


//////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsDone()
{
  Draw2D_Done();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
  DrawFill(RGBis(225, 225, 225), modeMUL);
  for (int i = 0; i < 3; i++) pcirc[i].regenerate(); 
  for (int i = 0; i < count; i++)
  { circ[i].regenerate();
    circ[i].draw(i, pcirc[0].xx, pcirc[0].yy, pcirc[1].xx, pcirc[1].yy, pcirc[2].xx, pcirc[2].yy, 500);
  };
  DrawPaint();
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Main()
{
  CommonsInit();                  //   ;  
  
  while (!keyExit) DrawAll();

  CommonsDone();                  //   ;
};
//////////////////////////////////////////////////////////////////////////////
int LetProcess()
{
  //MouseShowCursor = no;
  WinName = L"CG : SMotion";   //    
  VS_CColor = RGBis(0, 0, 0);    //   
  DrawSet(FullScreen);         //   ( )  , FullScreen   
  ProcessFunc = Main;           //    -  
  return on;
};
int Init_LetProcess = LetProcess();

